(function () {

  "use strict";

  // Ignore legacy browsers
  if (typeof document.querySelector === 'undefined') {
    window.lemonpi = [];
  }

  var defaults = {
    allowAllFieldNames: false,
    ignoreErrors: false,
    whitelistedDomain: '',
    fieldNames: ['id', 'available', 'title', 'imageUrl', 'clickUrl', 'category', 'basket', 'description', 'priceNormal', 'priceDiscount', 'logoUrl', 'stickerText', 'custom1', 'custom2', 'custom3', 'custom4']
  };

  function extend(){
    for(var i=1; i<arguments.length; i++)
      for(var key in arguments[i])
        if(arguments[i].hasOwnProperty(key))
          arguments[0][key] = arguments[i][key];
    return arguments[0];
  }

  var Scraper = function (fields, options) {
    this.fields = fields;
    this.options = extend({}, defaults, options);
    this.result = {};
    this.errors = [];

    this.verifyCanExecute();
    this.verifyRunningOnValidDomain(this.options.whitelistedDomain);
  };

  Scraper.getHash = getHash;
  Scraper.getHref = getHref;
  Scraper.getOrigin = getOrigin;
  Scraper.getPathName = getPathName;
  Scraper.isAbsoluteUrl = isAbsoluteUrl;
  Scraper.modifyUrlWithParameters = modifyUrlWithParameters;

  Scraper.prototype = {
    getText: function (v) {
      if(window.top.document.querySelector(v.selector).value){
        return window.top.document.querySelector(v.selector).value.trim();
      }
      return window.top.document.querySelector(v.selector).textContent.trim();
    },
    verifyRunningOnValidDomain: function(whitelistedDomain) {
      if (whitelistedDomain === '') return console.warn('This Smart Pixel can scrape from any page. For a more secure Smart Pixel, add a domain to the whitelistedDomain property of the Scraper configuration.');

      var isValidDomain = false;
      var currentUrl = location.host + location.pathname;

      if (typeof whitelistedDomain === 'string') {
        try {
          isValidDomain = new RegExp(whitelistedDomain).test(currentUrl);
        } catch(e) {
          throw new Error('The string \'' + whitelistedDomain + '\' is not a valid regular expression');
        }
      } else if (typeof whitelistedDomain === 'object') {
        isValidDomain = whitelistedDomain.test(currentUrl);
      }

      if(!isValidDomain) {
        throw new Error('You can\'t execute this Smart Pixel at this domain.');
      };
    },
    verifyCanExecute: function() {
      try {
        // Let's see if we can reach the DOM
        window.top.document.querySelector('body');
      } catch(e) {
        throw new Error('The Smart Pixel is executed in a unfriendly iframe and can\'t reach the DOM');
      }
    },
    getMatches: function (string, regex, index) {
      index = index || 1;
      var matches = [];
      var match;
      while ((match = regex.exec(string)) !== null) {
        matches.push(match[index]);
      }
      return matches;
    },
    createRegexFromString: function (regexString) {
      // If parameter already is a regexp object
      if (typeof regexString === 'object') return regexString;
      var flags = regexString.replace(/.*\/([gimy]*)$/, '$1');
      var pattern = regexString.replace(new RegExp('^/(.*?)/' + flags + '$'), '$1');
      return new RegExp(pattern, flags);
    },
    filterFieldNames: function (result) {
      var filteredResult = {};

      for(var key in this.options.fieldNames) {
        if (this.options.fieldNames.hasOwnProperty(key)) {
          var fieldName = this.options.fieldNames[key];
          if( result[fieldName] !== undefined ) {
            filteredResult[fieldName] = result[fieldName];
          }
        }
      };
      return filteredResult;
    },
    getField: function (fieldName) {
      if (this.result[fieldName]) {
        return this.result[fieldName];
      }
      var self = this;
      var result;
      var v = this.fields[fieldName];

      if (v === undefined) {
        return '';
      }

      if (typeof v === 'function') {
        result = v(self);
      } else {
        switch (v.type) {
          case 'url':
            var href = Scraper.getHref();

            if (v.query){
              href = Scraper.modifyUrlWithParameters(href, v.query);
            }

            if (v.prefix) {
              result = v.prefix + encodeURIComponent(href);
            } else {
              result = href;
            }
            break;

          case 'image':
            result = window.top.document.querySelector(v.selector).getAttribute('src');
            if (result && !Scraper.isAbsoluteUrl(result)) {
              if (result.indexOf('/') !== 0) {
                result = '/' + result;
              }
              result = Scraper.getOrigin() + result;
            }
            break;

          case 'regex':
            var regex = this.createRegexFromString(v.regex);
            result = this.getMatches(this.getText(v), regex)[0];
            break;

          case 'template':
            result = v.template.replace(/(\{\{.+?\}\})/g, function (a, b) {
              var fieldName = b.substr(2, b.length - 4);
              return self.getField(fieldName);
            });
            break;

          default:
            result = this.getText(v);
        }
      }

      if ((result === undefined || result === '') && v.required !== false) {
        this.errors.push(fieldName + ': ' + 'empty result');
      }

      if (v.test) {
        var testRegex = this.createRegexFromString(v.test);
        if (!testRegex.test(result)) {
          this.errors.push(fieldName + ': does not match test');
        }
      }

      if (v.post){
        result = v.post(result);
      }

      return result;
    },


    getResults: function () {
      var self = this;

      for(var fieldName in this.fields) {
        if (this.fields.hasOwnProperty(fieldName)) {
          self.result[fieldName] = self.getField(fieldName);
        }
      };

      if (this.errors.length > 0 && !this.options.ignoreErrors) {
        this.log(this.errors.join('\n'));

        return {};
      }

      if (!self.options.allowAllFieldNames) {
        self.result = self.filterFieldNames(self.result);
      }

      return self.result;
    },
    log: function (msg) {
      if (window.logerror) {
        logerror(msg);
      } else if (typeof console === "object" && typeof console.log === "function") {
        console.log(msg);
      }
    }
  };

  function getHref() {
    if (!window.location.origin) {
      window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port : '');
    }
    return window.location.href;
  }

  function getHash(){
    return window.location.hash;
  }

  function getPathName(){
    return window.location.pathname;
  }

  function getOrigin() {
    return window.location.origin;
  }

  function isAbsoluteUrl(url) {
    var pat = /^(?:[a-z]+:)?\/\//i;
    return pat.test(url);
  }

  function modifyUrlWithParameters(url, params) {
    var queryParams =  getUrlVars(url.split('#')[0]);
    var newUrl = Scraper.getOrigin()+Scraper.getPathName();
    var newParams = [];
    //
    for(var key in params) {
      if (params.hasOwnProperty(key)) {
        var value = params[key];
        if (typeof key === 'number') {
          key = value;
          value = params[key];
        }
        var queryValue = queryParams[key];
        if (!queryValue) {
          queryValue = value;
        }
        newParams.push(key+'='+queryValue);
      }
    };

    return newUrl+'?'+newParams.join('&');
  }

  function getUrlVars(url) {
    var vars = {}, hash;
    var hashes = url.slice(url.indexOf('?') + 1).split('&');
    for (var i = 0; i < hashes.length; i++) {
      hash = hashes[i].split('=');
      vars[hash[0]] = hash[1];
    }
    return vars;
  }

  window.Scraper = Scraper;

}());
