/* global describe,beforeEach,afterEach,it,jasmine,expect,Scraper, spyOn*/

describe('Scraper', function () {
  "use strict";

  beforeEach(function () {
    var f = jasmine.getFixtures();
    f.fixturesPath = 'base';
    f.load('test/fixtures/test.html');
  });

  afterEach(function () {
    var f = jasmine.getFixtures();
    f.cleanUp();
    f.clearCache();
  });

  it('should load fixture', function () {
    var fixture = $('.fixture');
    expect(fixture.length).toBe(1);
  });

  it('should set config on instance', function () {
    var scraper = new Scraper({});
    expect(scraper.fields).not.toBe(undefined);
  });

  it('should return results', function () {
    var scraper = new Scraper({});
    var results = scraper.getResults();
    expect(results).not.toBe(undefined);
  });

  it('should return text from an element by selector and type', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.simple-text'}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world');
  });

  it('should only return first occurence', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.simple-text-multiple'}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world');
  });

  it('should only return first occurence on input', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.simple-input'}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world');
  });

  it('should return multiple fields', function () {
    var scraper = new Scraper({
      custom1: {type: 'text', selector: '.simple-text'},
      custom2: {type: 'text', selector: '.simple-text2'}
    });
    var results = scraper.getResults();
    expect(results).toEqual({
      custom1: 'hello world',
      custom2: 'hello world 2'
    });
  });

  it('should return empty object if result is empty', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.non-existing-text'}});
    var results = scraper.getResults();
    expect(results).toEqual({});
  });

  it('should not return empty object if result is empty and field not required', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.non-existing-text', required:false}});
    var results = scraper.getResults();
    expect(results).not.toEqual({});
  });

  it('should log errors when fails', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.non-existing-text'}});
    spyOn(scraper, 'log');
    scraper.getResults();
    expect(scraper.log).toHaveBeenCalled();
  });

  it('should log empty result', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.non-existing-text'}});
    spyOn(scraper, 'log');
    scraper.getResults();
    expect(scraper.log).toHaveBeenCalledWith('custom1: empty result');
  });

  it('should call console.log if log error is nog available', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.non-existing-text'}});
    spyOn(console, 'log');
    scraper.getResults();
    expect(console.log).toHaveBeenCalledWith('custom1: empty result');
  });

  it('should test if url is absolute', function () {
    expect(Scraper.isAbsoluteUrl('http://www.bluemango.nl/image.png')).toBe(true);
    expect(Scraper.isAbsoluteUrl('https://www.bluemango.nl/image.png')).toBe(true);
    expect(Scraper.isAbsoluteUrl('//www.bluemango.nl/image.png')).toBe(true);
    expect(Scraper.isAbsoluteUrl('www.bluemango.nl/image.png')).toBe(false);
    expect(Scraper.isAbsoluteUrl('image.png')).toBe(false);
    expect(Scraper.isAbsoluteUrl('/image.png')).toBe(false);
  });

  it('should return src from image', function () {
    var scraper = new Scraper({custom1: {type: 'image', selector: 'img.simple-image'}});
    var results = scraper.getResults();
    expect(results.custom1).toEqual('http://www.bluemango.nl/image.jpg');
  });

  it('should return src from image', function () {
    var scraper = new Scraper({custom1: {type: 'image', selector: 'img.simple-image-no-image'}});
    spyOn(scraper, 'log');
    var results = scraper.getResults();
    expect(scraper.log).toHaveBeenCalledWith('custom1: empty result');
  });

  it('should prefix image if absolute but no origin', function () {
    var scraper = new Scraper({custom1: {type: 'image', selector: 'img.simple-image-absolute'}});
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    var results = scraper.getResults();
    expect(results.custom1).toEqual('http://www.bluemango.nl/image.jpg');
  });

  it('should prefix image if not absolute', function () {
    var scraper = new Scraper({custom1: {type: 'image', selector: 'img.simple-image-relative'}});
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    var results = scraper.getResults();
    expect(results.custom1).toEqual('http://www.bluemango.nl/image.jpg');
  });

  it('should select part of a text with a regex', function () {
    var scraper = new Scraper({custom1: {type: 'regex', regex:'/(hello)/g',selector: '.simple-text-regex'}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello');
  });

  it('should select part of a text with a regex and return the first match', function () {
    var scraper = new Scraper({custom1: {type: 'regex', regex:'/(hello)/g',selector: '.simple-text-regex2'}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello');
  });

  it('should combine field as a template', function () {
    var scraper = new Scraper({
      field1: {type: 'text', selector: '.simple-template-value1'},
      field2: {type: 'text', selector: '.simple-template-value2'},
      custom1: {type: 'template', template: '{{field1}} {{field2}}'}
    });
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world');
  });

  it('should test a result', function () {
    var scraper = new Scraper({text:{selector:'.simple-text', test:'/hello world2/g'}});
    var result = scraper.getResults();
    expect(result).toEqual({});
  });

  it('should log a failed test result', function () {
    var scraper = new Scraper({text:{selector:'.simple-text', test:'/hello world2/g'}});
    spyOn(scraper, 'log');
    scraper.getResults();
    expect(scraper.log).toHaveBeenCalledWith('text: does not match test');
  });

  it('should get the href', function () {
    var scraper = new Scraper({clickUrl:{type:'url'}});
    spyOn(Scraper, 'getHref').and.returnValue('http://www.bluemango.nl');
    var result = scraper.getResults();
    expect(result.clickUrl).toBe('http://www.bluemango.nl');
  });

  it('should prefix an url if prefix is set', function () {
    var scraper = new Scraper({clickUrl:{type:'url', prefix:'http://greenhousegroup.com/?redirect='}});
    spyOn(Scraper, 'getHref').and.returnValue('http://www.bluemango.nl');
    var result = scraper.getResults();
    expect(result.clickUrl).toBe('http://greenhousegroup.com/?redirect='+encodeURIComponent('http://www.bluemango.nl'));
  });

  it('should set a result based on a dictornary', function () {
    var scraper = new Scraper({custom1: {type: 'dictionary', selector: '.simple-text-dictionary', dictionary:{
      'hello world': '/foo bar/g'
    }}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world');
  });

  it('should set a result based on a dictornary', function () {
    var scraper = new Scraper({text: {type: 'dictionary', selector: '.simple-text-dictionary', dictionary:{
      'hello world': '/foo baz/g'
    }}});
    spyOn(scraper, 'log');
    scraper.getResults();
    expect(scraper.log).toHaveBeenCalledWith('text: empty result');
  });

  it('should return value when a function is used', function () {
    var scraper = new Scraper({custom1: function () {
      return '2';
    }});
    var result = scraper.getResults();
    expect(result.custom1).toBe('2');
  });

  it('should not return an unlisted property by default', function () {
    var scraper = new Scraper({text: {type: 'text', selector: '.simple-text'}});
    var results = scraper.getResults();
    expect(results.text).toBeUndefined();
  });

  it('should return an unlisted property when allowAllFieldNames is true', function () {
    var scraper = new Scraper({text: {type: 'text', selector: '.simple-text'}},{allowAllFieldNames:true});
    var results = scraper.getResults();
    expect(results.text).toBe('hello world');
  });

  it('should only return the url with the selected parameters', function () {
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    spyOn(Scraper, 'getPathName').and.returnValue('/');
    expect(Scraper.modifyUrlWithParameters('http://www.bluemango.nl/?param1=hello&param2=world', {param1:true})).toBe('http://www.bluemango.nl/?param1=hello');
    expect(Scraper.modifyUrlWithParameters('http://www.bluemango.nl/?param1=hello&param2=world', ['param1'])).toBe('http://www.bluemango.nl/?param1=hello');
    expect(Scraper.modifyUrlWithParameters('http://www.bluemango.nl/?param1=hello&param2=world', {param1:true, param3:'yeah'})).toBe('http://www.bluemango.nl/?param1=hello&param3=yeah');
    expect(Scraper.modifyUrlWithParameters('http://www.bluemango.nl/?param3=hello&param1=hello&param2=world', {param1:true,param3:true})).toBe('http://www.bluemango.nl/?param1=hello&param3=hello');
  });

  it('should only return the url with the selected parameters when scraped', function () {
    var scraper = new Scraper({clickUrl:{type:'url', query:['param1']}});
    spyOn(Scraper, 'getHref').and.returnValue('http://www.bluemango.nl?param1=hello&param2=world');
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    spyOn(Scraper, 'getPathName').and.returnValue('');
    var result = scraper.getResults();
    expect(result.clickUrl).toBe('http://www.bluemango.nl?param1=hello');
  });

  it('should only return the url with the selected parameters when scraped', function () {
    var scraper = new Scraper({clickUrl:{type:'url', query:['param1']}});
    spyOn(Scraper, 'getHref').and.returnValue('http://www.bluemango.nl?param1=hello&param2=world');
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    spyOn(Scraper, 'getPathName').and.returnValue('');
    var result = scraper.getResults();
    expect(result.clickUrl).toBe('http://www.bluemango.nl?param1=hello');
  });

  it('should remove hash from url', function () {
    var scraper = new Scraper({clickUrl:{type:'url', query:['param1']}});
    spyOn(Scraper, 'getHref').and.returnValue('http://www.bluemango.nl?param1=hello&param2=world#test=hash');
    spyOn(Scraper, 'getOrigin').and.returnValue('http://www.bluemango.nl');
    spyOn(Scraper, 'getPathName').and.returnValue('');
    var result = scraper.getResults();
    expect(result.clickUrl).toBe('http://www.bluemango.nl?param1=hello');
  });

  it('should be modify the result', function () {
    var scraper = new Scraper({custom1: {type: 'text', selector: '.simple-text', post: function (value) {
      return value+'!!!';
    }}});
    var results = scraper.getResults();
    expect(results.custom1).toBe('hello world!!!');
  });

});
